import { NS, Server } from '@ns'
import {selectAllServerList, ServerInfo} from "./servers"

export type Hackability = "hackable" | "rooted" | "impossible"

export function getHackability(ns: NS,hostname:string): Hackability{
    if (ns.hasRootAccess(hostname)) {
        return "rooted";
    }
    if (ns.getServerRequiredHackingLevel(hostname) > ns.getHackingLevel() ||
        ns.getServerNumPortsRequired(hostname) > hackablePorts(ns)) {
        return "impossible";
    }
    return "hackable";
}
export function getHackabilityServer(ns: NS,server:Server): Hackability{
    if (server.hasAdminRights) {
        return "rooted";
    }
    if (server.requiredHackingSkill > ns.getHackingLevel() ||
        server.numOpenPortsRequired > hackablePorts(ns)) {
        return "impossible";
    }
    return "hackable";
}

export function getLockSymbol(k:Hackability):string{
    switch (k) {
        case "rooted":
            return "🔓"
        case "hackable":
            return "🔐"
        case "impossible":
            return "🔒"
    }
}

type Hack = {
    file: string;
    exec: (ns: NS, target:string)=> void;
};

export const hacks: Hack[] = [
    { file: 'BruteSSH.exe', exec: (ns: NS, target: string) => ns.brutessh(target) },
    { file: 'FTPCrack.exe', exec: (ns: NS, target: string) => ns.ftpcrack(target) },
    { file: 'relaySMTP.exe', exec: (ns: NS, target: string) => ns.relaysmtp(target) },
    { file: 'HTTPWorm.exe', exec: (ns: NS, target: string) => ns.httpworm(target) },
    { file: 'SQLInject.exe', exec: (ns: NS, target: string) => ns.sqlinject(target) },
]

/**
 * @param {NS} ns
 */
export function hackablePorts(ns: NS): number {
    return hacks.map(x=>(ns.fileExists(x.file) ? 1 : 0)).reduce((x,y)=>x+y)
}

export function isHackableServer(ns: NS, target: Server): boolean{
    return target.numOpenPortsRequired <= hackablePorts(ns) &&
        target.hackDifficulty <= ns.getHackingLevel()
}
export function selectHackableServerList(ns: NS): ServerInfo[]{
    return selectAllServerList(ns).filter(x=>{
        return isHackableServer(ns,x)
    })
}

export function hackServer(ns:NS,target:Server);
export function hackServer(ns:NS,target:string);
export function hackServer(ns:NS,target:string|Server):void{
    const hostname = typeof target === "string" ? target : target.hostname
    hacks.forEach(x=>{
        if(ns.fileExists(x.file,"home")){
            x.exec(ns, hostname)
        }
    })
    ns.nuke(hostname)
}

/** @param {NS} ns */
// eslint-disable-next-line require-await
export async function main(ns: NS):Promise<void> {
    if(ns.args.length == 0){
        ns.tprint("argument required")
        ns.tprint("run cmd [target]")
        return
    }
    const target = ns.args[0]
    hackServer(target)
}

// eslint-disable-next-line @typescript-eslint/no-unused-vars
export function autocomplete(data : ServerData, args : string[]) : string[] {
    return [...data.servers]
}