import {NS} from "@ns";
import {hackablePorts, getHackability, getLockSymbol} from "./lib/hack";
import {parse} from "./lib/flag";


type ReportOption = {
    detail?: boolean;
}
/*
 * Utility functions that report serverStatus
 * and Hackability
 * @param {NS} ns
 */
// eslint-disable-next-line require-await
export async function main(ns: NS): Promise<void> {
    const flag = parse(ns.args);
    if (flag._.length == 0) {
        ns.tprint("argumented required");
        return;
    }
    const hostname = flag._[0];
    const detail = Boolean(flag.d) || Boolean(flag.detail);
    serverReport(ns, hostname,{
        detail: detail,
    });
}
/**
 * @param {NS} ns
 * @param {string} hostname
 */
export function serverReport(ns: NS, hostname: string, options?:ReportOption):void {
    options ??= {};
    const serverLock = getHackability(ns, hostname);
    ns.tprint(`${getLockSymbol(serverLock)} ${hostname}`);
    if(options.detail){
        const server = ns.getServer(hostname);
        const msg = ['',
            `hostname : ${server.hostname}(${server.ip})`,
            `🛡️${ns.nFormat(server.hackDifficulty,"0[.]000")}/${server.minDifficulty}(${server.baseDifficulty})`,
            `💸${ns.nFormat(server.moneyAvailable,"$0.000a")}/${ns.nFormat(server.moneyMax,"$0.000a")}`,
            `💾${server.ramUsed}GB/${server.maxRam}GB`,
            `backdoorInstalled \t: ${server.backdoorInstalled ? `🚪` : `❌`}`,
            `cpu \t\t\t: ${server.cpuCores}`,
            `growth \t\t\t: ${server.serverGrowth}`,
            `organization name\t: ${server.organizationName}`,
            `purchased \t\t: ${server.purchasedByPlayer}`,
            `required hacking skill\t: ${server.requiredHackingSkill}`,
            `ports \t\t\t: ${server.openPortCount}/${server.numOpenPortsRequired}`
        ].join("\n");
        ns.tprint(msg);
    }
    else{
        if (serverLock == "rooted") {
            ns.tprint(`🛡️${Math.round(ns.getServerSecurityLevel(hostname))}/${ns.getServerMinSecurityLevel(hostname)}`);
            ns.tprint(`💸${ns.nFormat(ns.getServerMoneyAvailable(hostname), "$0.000a")}/${ns.nFormat(ns.getServerMaxMoney(hostname), "$0.000a")}`);
        } else {
            ns.tprint(`Hack Level: ${ns.getServerRequiredHackingLevel(hostname)}`);
            ns.tprint(`Ports: ${ns.getServerNumPortsRequired(hostname)}`);
        }
    }
}
/**
 * @param {NS} ns
 */
export function serverHackStatus(ns: NS, server: string): string {
    if (ns.hasRootAccess(server)) {
        return "🔓";
    }
    if (ns.getServerRequiredHackingLevel(server) > ns.getHackingLevel() ||
        ns.getServerNumPortsRequired(server) > hackablePorts(ns)) {
        return "🔒";
    }
    return "🔐";
}

// eslint-disable-next-line @typescript-eslint/no-unused-vars
export function autocomplete(data : ServerData, args : string[]) : string[] {
    return [...data.servers]
}